package ch.ethz.se.rpnc.instruct;

import java.util.Hashtable;

import ch.ethz.se.rpnc.instruct.visitor.CalculatorInstructVisitor;
import ch.ethz.se.rpnc.ui.ResourceManager;

public class CalculatorInstructCommand implements ICalculatorInstruct {

	// Command type.
	private CalculatorCommandType type;

	// Private constructor to implement multi-ton pattern.
	private CalculatorInstructCommand(CalculatorCommandType t) {
		setType(t);
	}

	// Hashtable containing one command object for each command type.
	protected static Hashtable<CalculatorCommandType, CalculatorInstructCommand> commands;
	static {
		commands = new Hashtable<CalculatorCommandType, CalculatorInstructCommand>();
		for (CalculatorCommandType t : CalculatorCommandType.values()) {
			commands.put(t, new CalculatorInstructCommand(t));
		}
	}

	/**
	 * Get command object by type.
	 */
	public static CalculatorInstructCommand getInstructCommand(
			CalculatorCommandType t) {
		return commands.get(t);
	}

	/**
	 * Get command object according to its text representation.
	 */
	public static CalculatorInstructCommand getInstructCommand(String text) {
		for (CalculatorCommandType ct : CalculatorCommandType.values()) {
			if (text.equals(ResourceManager.getLocalizedUICommand(ct))) {
				return CalculatorInstructCommand.getInstructCommand(ct);
			}
		}
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * ch.ethz.se.rpnc.instruct.ICalculatorInstruct#accept(ch.ethz.se.rpnc.instruct
	 * .visitor.CalculatorInstructVisitor)
	 */
	@Override
	public void accept(CalculatorInstructVisitor v) throws Exception {
		v.visitCommand(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see ch.ethz.se.rpnc.instruct.ICalculatorInstruct#getInstructText()
	 */
	@Override
	public String getInstructText() {
		return getType().name().toLowerCase();
	}

	// Set command type.
	private void setType(CalculatorCommandType type) {
		this.type = type;
	}

	// Get command type.
	public CalculatorCommandType getType() {
		return type;
	}

}
