package ch.ethz.se.rpnc.ui;

import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;

import ch.ethz.se.rpnc.instruct.CalculatorCommandType;

public class ResourceManager {

	/**
	 * Get the name of effective language.
	 */
	public static String getLanguageName() {
		return getLocalizedUICommand(getLanguageType());
	}

	/**
	 * Get the type of effective language.
	 */
	public static CalculatorCommandType getLanguageType() {
		return resourceManager.curLanguageType;
	}

	/**
	 * Set effective language.
	 * 
	 * @param aKey
	 *            A key specifying the language.
	 */
	public static void setLanguage(CalculatorCommandType t) throws Exception {
		resourceManager.setLanguageByType(t);
	}

	/**
	 * Query a text in effective language.
	 */
	public static String getLocalizedUIMessage(UIMessage msg) {
		return resourceManager.getResourceBundle().getString(msg.name().toLowerCase());
	}
	
	public static String getLocalizedUICommand(CalculatorCommandType ct){
		return resourceManager.getResourceBundle().getString(ct.name().toLowerCase());
	}

	// Resource manager singleton.
	private static ResourceManager resourceManager = new ResourceManager("Calculator");

	// File name of the resource bundle.
	private final String resource_bundle_name;

	// Resource bundle currently in use.
	private ResourceBundle curResourceBundle;

	// Type of effective language.
	private CalculatorCommandType curLanguageType;

	// Hashtable of locale code, language type as the hash key and locale code
	// as value.
	private Hashtable<CalculatorCommandType, Locale> localeCodeTable;

	// Resource manager uses English as default interface language.
	private ResourceManager(String bundleName) {
		resource_bundle_name = bundleName;

		localeCodeTable = new Hashtable<CalculatorCommandType, Locale>();
		localeCodeTable.put(CalculatorCommandType.CMD_ENGLISH, new Locale("en",
				"US"));
		localeCodeTable.put(CalculatorCommandType.CMD_DEUTSCH, new Locale("de",
				"CH"));

		try {
			setLanguageByType(CalculatorCommandType.CMD_ENGLISH);
		} catch (Exception e) {
		}
	}

	// Get resource bundle.
	private ResourceBundle getResourceBundle() {
		return curResourceBundle;
	}

	// Select interface language.
	// Language is only changed if the corresponding resource bundle is
	// available.
	private void setLanguageByType(CalculatorCommandType langType)
			throws Exception {
		if (!langType.equals(curLanguageType)) {
			boolean isSuccessful = true;
			if (localeCodeTable.containsKey(langType)) {
				try {
					curResourceBundle = ResourceBundle
							.getBundle(resource_bundle_name, localeCodeTable
									.get(langType));
					curLanguageType = langType;
				} catch (Exception e) {
					isSuccessful = false;
				}
			} else
				isSuccessful = false;

			if (!isSuccessful)
				throw new Exception("\n"
						+ getLocalizedUIMessage(UIMessage.MSG_MISSING_RESOURCE) 
						+ "\n\t" + langType);
		}
	}

}
